/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_ALIGNMENT_AMBIGUOUSDNASCORINGMATRIX_HH__
#define __DEWEYLAB_BIO_ALIGNMENT_AMBIGUOUSDNASCORINGMATRIX_HH__

#include <istream>

#include "deweylab/bio/alphabet/AmbiguousNucleotide.hh"
#include "deweylab/bio/alignment/AlphabetScoringMatrix.hh"
#include "deweylab/bio/alignment/DNAScoringMatrix.hh"

namespace deweylab {
namespace bio {
namespace alignment {

using bio::alphabet::AMBIGUOUS_DNA;

template<typename Score>
class AmbiguousDNAScoringMatrix : public AlphabetScoringMatrix<Score> {
private:
	void init(const DNAScoringMatrix<Score>& dnaMatrix);
		
public:
	AmbiguousDNAScoringMatrix();
	AmbiguousDNAScoringMatrix(const DNAScoringMatrix<Score>& dnaMatrix);
	AmbiguousDNAScoringMatrix(Score match, Score mismatch);
		
};

template<typename Score>
void AmbiguousDNAScoringMatrix<Score>::init(const DNAScoringMatrix<Score>& dnaMatrix) {
	for (size_t i = 0; i < AMBIGUOUS_DNA.getSize(); ++i) {
		char ambiguousChar1 = AMBIGUOUS_DNA.decode(i);
		std::string chars1 = AMBIGUOUS_DNA.getPossibleChars(ambiguousChar1);
		for (size_t j = 0; j < AMBIGUOUS_DNA.getSize(); ++j) {
			char ambiguousChar2 = AMBIGUOUS_DNA.decode(j);
			std::string chars2 = AMBIGUOUS_DNA.getPossibleChars(ambiguousChar2);
			Score total = 0;
			typedef std::string::const_iterator CharIt;
			for (CharIt c1 = chars1.begin(); c1 != chars1.end(); ++c1) {
				for (CharIt c2 = chars2.begin(); c2 != chars2.end(); ++c2) {
					total += dnaMatrix.getCharScore(*c1, *c2);
				}
			}
				
			this->setCharScore(ambiguousChar1, ambiguousChar2,
                               total / static_cast<int>(chars1.size() * chars2.size()));
		}
	}
}
	
template<typename Score>
AmbiguousDNAScoringMatrix<Score>::AmbiguousDNAScoringMatrix()
	: AlphabetScoringMatrix<Score>(AMBIGUOUS_DNA) {
}

template<typename Score>
AmbiguousDNAScoringMatrix<Score>::AmbiguousDNAScoringMatrix(const DNAScoringMatrix<Score>& dnaMatrix)
	: AlphabetScoringMatrix<Score>(AMBIGUOUS_DNA) {
	init(dnaMatrix);
}
	
template<typename Score>
AmbiguousDNAScoringMatrix<Score>::AmbiguousDNAScoringMatrix(Score match, Score mismatch)
	: AlphabetScoringMatrix<Score>(AMBIGUOUS_DNA) {
	DNAScoringMatrix<Score> dnaMatrix(match, mismatch);
	init(dnaMatrix);
}

	
} }

};
#endif // __DEWEYLAB_BIO_ALIGNMENT_AMBIGUOUSDNASCORINGMATRIX_HH__
