/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSAMINOACID_HH__
#define __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSAMINOACID_HH__

#include <string>
#include <vector>

#include "deweylab/bio/alphabet/AmbiguousAlphabet.hh"
#include "deweylab/bio/alphabet/AminoAcid.hh"

namespace deweylab {
namespace bio {
namespace alphabet {

class AmbiguousAminoAcid : public AminoAcid, public AmbiguousAlphabet {
private:
	const std::vector<std::string> ambiguities;
	
public:

	template<typename Iterator>
	AmbiguousAminoAcid(const std::string chars,
					   Iterator name_iter,
					   Iterator short_name_iter,
					   Iterator ambiguity_iter);
	
	std::string getPossibleChars(const char c) const;
};
	
template<typename Iterator>
AmbiguousAminoAcid::AmbiguousAminoAcid(const std::string chars,
									   Iterator name_iter,
									   Iterator short_name_iter,
									   Iterator ambiguity_iter)
	: AminoAcid(chars, name_iter, short_name_iter),
	  ambiguities(ambiguity_iter, ambiguity_iter + chars.size()) {
}

inline std::string AmbiguousAminoAcid::getPossibleChars(const char c) const {
	return ambiguities[AminoAcid::encode(c)];
}

const std::string AMBIGUOUS_PROTEIN_AMBIGUITIES[] =
{"A", "C", "D", "E", "F", "G", "H", "I", "K", "L",
 "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y",
 "DN",
 "ACDEFGHIKLMNPQRSTVWY",
 "EQ"};

const std::string AMBIGUOUS_PROTEIN_SHORT_NAMES[] =
{"Ala", "Cys", "Asp", "Glu",
 "Phe", "Gly", "His", "Ile", "Lys",
 "Leu", "Met", "Asn", "Pro", "Gln",
 "Arg", "Ser", "Thr", "Val", "Trp",
 "Tyr", "Asx", "Xaa", "Glx"};

const std::string AMBIGUOUS_PROTEIN_NAMES[] =
{"Alanine",
 "Cysteine",
 "Aspartic acid",
 "Glutamic acid",
 "Phenylalanine",
 "Glycine",
 "Histidine",
 "Isoleucine",
 "Lysine",
 "Leucine",
 "Methionine",
 "Asparagine",
 "Proline",
 "Glutamine",
 "Arginine",
 "Serine",
 "Threonine",
 "Valine",
 "Tryptophan",
 "Tyrosine",
 "Aspartic acid or Asparagine",
 "Any",
 "Glutamine or Glutamic acid"};

const AmbiguousAminoAcid
AMBIGUOUS_PROTEIN("ACDEFGHIKLMNPQRSTVWYBXZ",
				  AMBIGUOUS_PROTEIN_NAMES,
				  AMBIGUOUS_PROTEIN_SHORT_NAMES,
				  AMBIGUOUS_PROTEIN_AMBIGUITIES);

} } }

#endif // __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSAMINOACID_HH__
