/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSNUCLEOTIDE_HH__
#define __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSNUCLEOTIDE_HH__

#include <string>
#include <vector>

#include "deweylab/bio/alphabet/AmbiguousAlphabet.hh"
#include "deweylab/bio/alphabet/Nucleotide.hh"

namespace deweylab {
namespace bio {
namespace alphabet {

class AmbiguousNucleotide : public Nucleotide, public AmbiguousAlphabet {
private:
	const std::vector<std::string> ambiguities;
		
public:

	template<typename Iterator>
	AmbiguousNucleotide(const std::string chars,
						const std::string complement,
						Iterator name_iter,
						Iterator ambiguity_iter);

	std::string getPossibleChars(const char c) const;
};

template<typename Iterator>
AmbiguousNucleotide::AmbiguousNucleotide(const std::string chars,
										 const std::string complement,
										 Iterator name_iter,
										 Iterator ambiguity_iter) :
	Nucleotide(chars, complement, name_iter), 
	ambiguities(ambiguity_iter, ambiguity_iter + chars.size()) {
}

inline std::string AmbiguousNucleotide::getPossibleChars(const char c) const {
	return ambiguities[Nucleotide::encode(c)];
}
	
const std::string AMBIGUOUS_DNA_AMBIGUITIES[] =
{"A", "C", "G", "T",
 "ACGT",
 "AC", "AG", "AT", "CG", "CT", "GT",
 "ACG", "ACT", "AGT", "CGT"};

const std::string AMBIGUOUS_RNA_AMBIGUITIES[] =
{"A", "C", "G", "U",
 "ACGU",
 "AC", "AG", "AU", "CG", "CU", "GU",
 "ACG", "ACU", "AGU", "CGU"};

const std::string AMBIGUOUS_DNA_NAMES[] =
{"Adenine", "Cytosine", "Guanine", "Thymine",
 "Any",
 "AC", "Purine", "Weak", "Strong", "Pyrimidine", "GT",
 "Not T", "Not G", "Not C", "Not A"};

const std::string AMBIGUOUS_RNA_NAMES[] =
{"Adenine", "Cytosine", "Guanine", "Uracil",
 "Any",
 "AC", "Purine", "Weak", "Strong", "Pyrimidine", "GU",
 "Not U", "Not G", "Not C", "Not A"};

const AmbiguousNucleotide
AMBIGUOUS_DNA("ACGTNMRWSYKVHDB",
			  "TGCANKYWSRMBDHV",
			  AMBIGUOUS_DNA_NAMES,
			  AMBIGUOUS_DNA_AMBIGUITIES);

const AmbiguousNucleotide
GENOME_DNA("ACGTN",
		   "TGCAN",
		   AMBIGUOUS_DNA_NAMES,
		   AMBIGUOUS_DNA_AMBIGUITIES);

const AmbiguousNucleotide
AMBIGUOUS_RNA("ACGUNMRWSYKVHDB",
			  "UGCANKYWSRMBDHV",
			  AMBIGUOUS_RNA_NAMES,
			  AMBIGUOUS_RNA_AMBIGUITIES);

} } }

#endif // __DEWEYLAB_BIO_ALPHABET_AMBIGUOUSNUCLEOTIDE_HH__
