/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_FASTQ_INPUTSTREAM_HH__
#define __DEWEYLAB_BIO_FORMATS_FASTQ_INPUTSTREAM_HH__

#include "Constants.hh"
#include "Record.hh"
#include "QualityScoreEncoding.hh"
#include "deweylab/util/io/line/InputStream.hh"
#include "deweylab/util/stl.hh"

#include <iosfwd>
#include <string>

namespace deweylab {
namespace bio {
namespace formats {
namespace fastq {

class InputStream : private Constants {
public:
    InputStream(std::istream& stream,
                const QualityScoreEncoding* encoding = DEFAULT_QUALITY_SCORE_ENCODING);
		
    InputStream& operator>>(Record& rec);
    operator bool();
    bool operator!();
 	
    typedef Record ValueType;

private:
    util::io::line::InputStream stream_;
    const QualityScoreEncoding* qualities_encoding_;
    bool at_end_;
    std::string last_line_;
};

typedef util::stl::InputStreamIterator<InputStream, Record> InputStreamIterator;

}
}
}
}

#endif // __DEWEYLAB_BIO_FORMATS_FASTQ_INPUTSTREAM_HH__
