/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_FASTQ_OUTPUTSTREAM_HH__
#define __DEWEYLAB_BIO_FORMATS_FASTQ_OUTPUTSTREAM_HH__

#include "Constants.hh"
#include "Record.hh"
#include "deweylab/util/stl.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace fastq {

class OutputStream : private Constants {
public:
	OutputStream(std::ostream& stream,
                 const QualityScoreEncoding* encoding);

	void set_line_width(size_t line_width) { line_width_ = line_width; }
	void set_repeat_title(bool repeat_title) { repeat_title_ = repeat_title; }

	OutputStream& operator<<(const Record& r);

    typedef Record ValueType;

private:
	std::ostream& stream_;
    const QualityScoreEncoding* qualities_encoding_;
	size_t line_width_;
    bool repeat_title_;
};

typedef util::stl::OutputStreamIterator<OutputStream, Record> OutputStreamIterator;

}
}
}
}

#endif // __DEWEYLAB_BIO_FORMATS_FASTQ_OUTPUTSTREAM_HH__
