/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_NEWICK_INPUTSTREAM_HH__
#define __DEWEYLAB_BIO_FORMATS_NEWICK_INPUTSTREAM_HH__

#include <iostream>
#include <string>
#include <stack>

#include "deweylab/bio/phylogenetic/Tree.hh"
#include "deweylab/util/stl.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace newick {

using phylogenetic::TreePtr;

struct TreeGrammar;
typedef std::stack<TreePtr> TreeStack;

class InputStream {
public:
    InputStream(std::istream& stream);
    InputStream(const std::string& s);
    InputStream(const InputStream& other);
	~InputStream();

    InputStream& operator>>(TreePtr& tree);
    operator bool();
    bool operator!();
	
    typedef TreePtr ValueType;

private:
	void init();
	
	std::string s;
	std::string::const_iterator curr;
	bool at_end;
	TreeStack stack;
	TreeGrammar* grammar;
};

typedef util::stl::InputStreamIterator<InputStream, TreePtr> InputStreamIterator;

} } } }

#endif // __DEWEYLAB_BIO_FORMATS_NEWICK_INPUTSTREAM_HH__
