/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_NEWICK_OUTPUTSTREAM_HH__
#define __DEWEYLAB_BIO_FORMATS_NEWICK_OUTPUTSTREAM_HH__

#include <iosfwd>

#include "deweylab/bio/phylogenetic/Tree.hh"
#include "deweylab/bio/phylogenetic/TreeVisitor.hh"
#include "deweylab/util/stl.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace newick {

using phylogenetic::ConstTreePtr;

class OutputStream : private phylogenetic::ConstTreeVisitor {
public:
	OutputStream(std::ostream& stream,
				 bool write_branch_lengths=true);

	void setWriteBranchLengths(bool b);

	OutputStream& operator<<(ConstTreePtr t);

    typedef phylogenetic::ConstTreePtr ValueType;
private:
	void preVisitNode(const ConstTreePtr& t);
	void postVisitNode(const ConstTreePtr& t);
	void postVisitTree(const ConstTreePtr& tree);
	void preVisitEdge(const ConstTreePtr& parent,
					  const ConstTreePtr& child);
	void postVisitEdge(const ConstTreePtr& parent,
					   const ConstTreePtr& child);
	
	std::ostream& stream;
	bool write_branch_lengths;
	bool add_comma;
};

typedef util::stl::OutputStreamIterator<OutputStream, ConstTreePtr> OutputStreamIterator;

} } } }

#endif // __DEWEYLAB_BIO_FORMATS_NEWICK_OUTPUTSTREAM_HH__
