/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "InputStream.hh"

#include <istream>

#include "deweylab/util/string.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace qual {

void parse_qualities(const std::string& s,
                     std::vector<PhredScore>& qualities) {
    qualities.clear();
    std::istringstream stream(s);
    unsigned int score;
    while (true) {
        stream >> score;
        if (!stream) break;
        qualities.push_back(score);
    }
    if (not stream.eof()) {
        throw FormatError("Invalid quality scores:\n" + s);
    }
}

InputStream::InputStream(std::istream& stream,
                         const size_t bufferSize) 
    : fasta_stream_(stream, bufferSize) {
    fasta_stream_.setKeepWhitespace(true);
}

InputStream& InputStream::operator>>(Record& rec) {
    fasta::Record fasta_rec;
    fasta_stream_ >> fasta_rec;
    if (fasta_stream_) {
        rec.id = fasta_rec.id;
        rec.description = fasta_rec.description;
        parse_qualities(fasta_rec.sequence, rec.qualities);
    }
    return *this;
}

InputStream::operator bool() const { return fasta_stream_; }

bool InputStream::operator!() const { return !fasta_stream_; }

} } } }
