/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_SDB_RECORD_HH__
#define __DEWEYLAB_BIO_FORMATS_SDB_RECORD_HH__

#include "deweylab/bio/alphabet/AmbiguousNucleotide.hh"

#include "boost/cstdint.hpp"

namespace deweylab {
namespace bio {
namespace formats {
namespace sdb {

class DB;
struct RecordSorter;
	
class Record {
public:
    Record();
		
    const std::string& getTitle() const;
    uint32_t getRecNum() const;
    uint32_t getLength() const;
    std::string getSeq();
    std::string getSeq(const uint32_t start,
					   const uint32_t end,
					   const char strand='+',
					   const alphabet::Nucleotide& alphabet=alphabet::AMBIGUOUS_DNA);

    bool operator==(const Record& other) const;
		
private:
    Record(DB* db);
    void write() const;
    void read();
    uint32_t getBinarySize() const;
    std::string readSeq(const uint32_t offset,
						const uint32_t length);
    void bufferSeq(const uint32_t offset,
				   std::vector<char>& buffer);
    size_t getCompressedLength() const;

    DB* db;
    std::string seq;
		
    std::string title;
    uint32_t recNum;
    uint32_t leftSize;
    uint32_t rightSize;
    uint32_t seqLength;
    off_t seqPos;
    unsigned char compressed;

    friend class DB;
    friend struct RecordSorter;
};

} } } }

#endif // __DEWEYLAB_BIO_FORMATS_SDB_RECORD_HH__
