# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
from __future__ import generators

_FIELD_NAMES = ("recClass",
                "tStart",
                "tSize",
                "qChrom",
                "orientation",
                "qStart",
                "qSize")
_FIELD_TYPES = (str, int, int, str, str, int, int)

_ATTRIBUTE_TYPES = {"id": int,
                    "score": int,
                    "ali": int,
                    "qFar": int,
                    "qOver": int,
                    "qDup": int,
                    "type": str,
                    "tN": int,
                    "qN": int,
                    "tR": int,
                    "qR": int,
                    "tNewR": int,
                    "qNewR": int,
                    "tOldR": int,
                    "qOldR": int,
                    "tTrf": int,
                    "qTrf": int}

class Record:
    def __init__(self, **fields):
        self.__dict__["fields"] = fields

    def _fieldsString(self):
        return ' '.join([str(self.fields[k]) for k in _FIELD_NAMES])

    def _attributesString(self):
        return ' '.join([' '.join(map(str, pair))
                         for pair in self.attributes.items()])

    def __getattr__(self, key):
        return self.fields[key]

    def __setattr__(self, key, value):
        self.fields[key] = value

    def __str__(self):
        s = ' ' * self.level
        s += self._fieldsString()
        if self.attributes:
            s += ' ' + self._attributesString()
        return s

def Iterator(stream):
    for line in stream:
        stripped = line.lstrip()
        level = len(line) - len(stripped)
        tokens = stripped.split()

        if level == 0:
            tChrom, tChromSize = tokens[1], int(tokens[2])
            continue

        fields = dict([(key, t(val)) for key, t, val
                       in zip(_FIELD_NAMES, _FIELD_TYPES, tokens[:7])])
        pairs = [(tokens[i], tokens[i + 1]) for i in xrange(7, len(tokens), 2)]
        attributes = dict([(key, _ATTRIBUTE_TYPES[key](value)) for
                           key, value in pairs])

        yield Record(level=level,
                     tChrom=tChrom,
                     tChromSize=tChromSize,
                     attributes=attributes,
                     **fields)
