# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import operator
import math

def product(lst): return reduce(operator.mul, lst, 1)

def mean(lst): return float(sum(lst)) / len(lst)

def midval(lst, low, high):
    numelts = high - low
    if numelts % 2 == 1:
        return lst[low + numelts / 2]
    else:
        return mean(lst[low - 1 + numelts / 2 : low + 1 + numelts / 2])

def median(lst):
    return midval(lst, 0, len(lst))

def n50(lst):
    fiftypct = sum(lst) / 2.0
    partialsum = 0
    i = len(lst)
    while partialsum < fiftypct:
        i -= 1
        partialsum += lst[i]
    return lst[i]

def firstquartile(lst):
    return midval(lst, 0, len(lst) / 2)

def thirdquartile(lst):
    return midval(lst, len(lst) / 2, len(lst))

def counts(lst):
    cts = {}
    for x in lst:
        cts[x] = cts.get(x, 0) + 1
    return cts
    
def mode(lst):
    cts = counts(lst)
    countpairs = zip(cts.values(), cts.keys())
    countpairs.sort()
    maxcount = countpairs[-1][0]
    return ([key for ct, key in countpairs if ct == maxcount], maxcount)

def variance(lst):
    m = mean(lst)
    return sum([(x - m)**2 for x in lst]) / (len(lst) - 1)

def stddev(lst):
    return math.sqrt(variance(lst))
