#define BOOST_TEST_MODULE qual

#include <boost/test/unit_test.hpp>
#include "deweylab/bio/formats/qual.hh"

#include <string>
#include <vector>

using namespace deweylab::bio::formats::qual;

BOOST_AUTO_TEST_CASE( one_record ) {
    std::string id("simple");
    std::string description("description");
    std::vector<PhredScore> qualities;
    for (size_t i = 0; i < 40; ++i) {
        qualities.push_back(i);
    }
    Record simple_record(id, description, qualities);

    std::string simple =
        ">simple description\n"
        "0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19\n"
        "20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39\n";
    std::istringstream stream(simple);
    InputStream qual_stream(stream);
    Record record;
    qual_stream >> record;

    BOOST_CHECK_EQUAL(simple_record.id, record.id);
    BOOST_CHECK_EQUAL(simple_record.description, record.description);
    BOOST_CHECK(simple_record.qualities == record.qualities);
    BOOST_CHECK(qual_stream);
    qual_stream >> record;
    BOOST_CHECK(!qual_stream);

    std::ostringstream outstream;
    OutputStream qual_outstream(outstream);
    qual_outstream.setLineWidth(20);
    qual_outstream << simple_record;
    std::string outstring(outstream.str());
    BOOST_CHECK_EQUAL(simple, outstring);
}

BOOST_AUTO_TEST_CASE( empty ) {
    std::string empty = "";
    std::istringstream stream(empty);
    InputStream qual_stream(stream);
    Record record;
    BOOST_CHECK(qual_stream);
    qual_stream >> record;
    BOOST_CHECK(!qual_stream);
    
    std::ostringstream outstream;
    OutputStream qual_outstream(outstream);
    qual_outstream.setLineWidth(20);
    std::string outstring(outstream.str());
    BOOST_CHECK_EQUAL(empty, outstring);
}

BOOST_AUTO_TEST_CASE( multi_records ) {
    std::string id("simple");
    std::string description("description");
    std::vector<PhredScore> qualities;
    for (size_t i = 0; i < 40; ++i) {
        qualities.push_back(i);
    }
    Record simple_record1(id + "1", description, qualities);
    Record simple_record2(id + "2", description, qualities);
    Record simple_record3(id + "3", description, qualities);

    std::string simple =
        ">simple1 description\n"
        "0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19\n"
        "20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39\n"
        ">simple2 description\n"
        "0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19\n"
        "20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39\n"
        ">simple3 description\n"
        "0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19\n"
        "20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39\n";
    std::istringstream stream(simple);
    InputStream qual_stream(stream);
    Record record;
    qual_stream >> record;
    BOOST_CHECK_EQUAL(simple_record1.id, record.id);
    BOOST_CHECK_EQUAL(simple_record1.description, record.description);
    BOOST_CHECK(simple_record1.qualities == record.qualities);

    qual_stream >> record;
    BOOST_CHECK_EQUAL(simple_record2.id, record.id);
    BOOST_CHECK_EQUAL(simple_record2.description, record.description);
    BOOST_CHECK(simple_record2.qualities == record.qualities);

    qual_stream >> record;
    BOOST_CHECK_EQUAL(simple_record3.id, record.id);
    BOOST_CHECK_EQUAL(simple_record3.description, record.description);
    BOOST_CHECK(simple_record3.qualities == record.qualities);

    BOOST_CHECK(qual_stream);
    qual_stream >> record;
    BOOST_CHECK(!qual_stream);

    std::ostringstream outstream;
    OutputStream qual_outstream(outstream);
    qual_outstream.setLineWidth(20);
    qual_outstream << simple_record1;
    qual_outstream << simple_record2;
    qual_outstream << simple_record3;
    std::string outstring(outstream.str());
    BOOST_CHECK_EQUAL(simple, outstring);
}
