/* Copyright (c) 2010
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include "deweylab/bio/formats/fastq.hh"
#include "deweylab/util/options.hh"

#include <algorithm>
#include <iostream>

int main(int argc, const char* argv[]) {
    size_t line_width;
    bool repeat_title;
    std::string input_encoding_name;
    std::string output_encoding_name;

    deweylab::util::options::Parser parser("< fastq input",
                                           "Reformat FASTQ records");
    parser.addStoreOpt('w', "line-width",
                       "number of characters at which sequence is wrapped",
                       line_width = 0);
    parser.addStoreTrueOpt('r', "repeat-title",
                           "repeat title line for quality score sequence",
                           repeat_title = false);
    parser.addStoreOpt('i', "input-encoding",
                       "Input quality score encoding scheme",
                       input_encoding_name = "sanger");
    parser.addStoreOpt('o', "output-encoding",
                       "Output quality score encoding scheme",
                       output_encoding_name = "sanger");
	parser.parse(argv, argv + argc);
    
	try {
        using namespace deweylab::bio::formats::fastq;
        const QualityScoreEncoding* input_encoding = get_quality_score_encoding(input_encoding_name);
        const QualityScoreEncoding* output_encoding = get_quality_score_encoding(output_encoding_name);
        InputStream in_stream(std::cin, input_encoding);
        OutputStream out_stream(std::cout, output_encoding);
        out_stream.set_line_width(line_width);
        out_stream.set_repeat_title(repeat_title);

        InputStreamIterator in_begin(in_stream), in_end;
        OutputStreamIterator out_begin(out_stream);
        std::copy(in_begin, in_end, out_begin);

    } catch (const std::runtime_error& e) {
		std::cerr << "Error: " << e.what() << '\n';
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
