/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <iterator>

#include "deweylab/bio/phylogenetic/Tree.hh"
#include "deweylab/bio/formats/newick.hh"
#include "deweylab/util/options.hh"
using namespace deweylab;
using namespace bio;
using namespace formats;
using bio::phylogenetic::TreePtr;

class TreeSizer {
private:
	bool all;
public:
	TreeSizer(bool all) : all(all) {}
	size_t operator()(const TreePtr& tree) const {
		return all ? tree->getNumNodes() : tree->getNumLeaves();
	}
};

int main(int argc, const char* argv[]) {
	// Increase speed of input/output to standard streams
	std::ios::sync_with_stdio(false);

	// Initialize options to defaults
	bool all = false;

	// Parse command line
	util::options::Parser parser("< newickInput",
								 "Output the sizes of the trees in the input");
	parser.addStoreTrueOpt('a', "all",
						   "Count all nodes, not just leaves", all);
	parser.parse(argv, argv + argc);

	try {
		newick::InputStream in_stream(std::cin);
		newick::InputStreamIterator in_begin(in_stream), in_end;
		std::ostream_iterator<size_t> out(std::cout, "\n");

		std::transform(in_begin, in_end, out, TreeSizer(all));

	} catch (const std::runtime_error& e) {
		std::cerr << "Error: " << e.what() << '\n';
		return EXIT_FAILURE;
	}
	
	return EXIT_SUCCESS;
}
