/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "deweylab/bio/genome/SequenceDB.hh"
#include "deweylab/bio/alphabet/AmbiguousNucleotide.hh"

namespace deweylab {
namespace bio {
namespace genome {

std::string SequenceDB::getSubseq(const std::string& title,
                                  const Position start,
                                  const Position end) {
	return getSeq(title).substr(start, end - start);
}

std::string SequenceDB::getStrandSubseq(const std::string& title,
                                        const Position start,
                                        const Position end,
                                        const Strand strand) {
	std::string seq(getSubseq(title, start, end));
	if (strand == '-') {
		alphabet::AMBIGUOUS_DNA.reverseComplementInPlace(seq);
	}
	return seq;	
}

std::string SequenceDB::getIntervalSubseq(const Interval& i) {
	return getStrandSubseq(i.getChrom(), i.getStart(), i.getEnd(), i.getStrand());
}

} } }
