# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
from deweylab.bio.formats import fasta

def getDBSeq(db, chrom, start, end,
             strand='+', masked=0, hardMasked=0):
    options = []
    if hardMasked:
        options.append('-h')
    elif not masked:
        options.append('-u')
    options = ' '.join(options)

    cmd = ("dl-sdb-export %(options)s %(db)s %(chrom)s " +
           "%(start)d %(end)d %(strand)s") % vars()
    stream = os.popen(cmd, 'r')
    return fasta.Reader(stream).next().sequence

class Interval:
    def __init__(self, chrom, start, end, strand='+'):
        self.chrom = chrom
        self.start = start
        self.end = end
        self.strand = strand

    def __lt__(self, other):
        return self.chrom < other.chrom \
               or (self.chrom == other.chrom and self.start < other.start)

    def __sub__(self, other):
        return Interval(self.chrom, other.end, self.start, self.strand)

    def __str__(self):
        return "%s:%d-%d%s" % (self.chrom, self.start, self.end, self.strand)

    def contains(self, other):
        return (self.chrom == other.chrom and
                self.start <= other.start and
                self.end >= other.end)

    def overlaps(self, other):
        return (self.chrom == other.chrom and
                self.start < other.end and
                self.end > other.start)

    def getLength(self):
        return self.end - self.start
