/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string>
#include <iostream>

#include "deweylab/bio/formats/fasta.hh"
#include "deweylab/util/options.hh"
using namespace deweylab;
using namespace bio::formats::fasta;

struct RecordConcatenator {
	Record rec;
	RecordConcatenator(const std::string& title) : rec(title, "", "") {}
	void operator()(const Record& other) { rec.sequence.append(other.sequence); }
};

int main(int argc, const char* argv[]) {
	// Increase speed of input/output to standard streams
	std::ios::sync_with_stdio(false);

	// Intialize options and arguments
	std::string title;
	
	util::options::Parser parser("< multi.fasta > single.fasta", "");
	parser.addStoreArg("title", "The title to give to the merged fasta record",
					   title);
	parser.parse(argv, argv + argc);
	
	try {
		InputStream in_stream(std::cin);
		InputStreamIterator in_begin(in_stream), in_end;
		OutputStream out_stream(std::cout);
		RecordConcatenator concatenator(title);
		
		out_stream << std::for_each(in_begin, in_end, concatenator).rec;
		
	} catch (const std::runtime_error& e) {
		std::cerr << "Error: " << e.what() << '\n';
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
