#!/usr/bin/env python

# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
import random
from deweylab.bio.formats import fasta
from optparse import OptionParser

usage = "usage: %prog [options] < fastaInput"
optparser = OptionParser(usage)
optparser.add_option("-s", "--seed", type="int", dest="seed",
                     help="seed to random number generator",
                     default=None)
optparser.add_option("-n", type="int", dest="number",
                     help="number of records to randomly output",
                     default=10, metavar="NUMBER")
(options, args) = optparser.parse_args()
if len(args) != 0:
    optparser.error("incorrect number of arguments")

recs = fasta.Reader(sys.stdin).read_recs()

random.seed(options.seed)
random.shuffle(recs)
writer = fasta.Writer(sys.stdout)
for rec in recs[:options.number]:
    writer.write(rec)

