/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "deweylab/app/FastaPipe.hh"
#include "deweylab/util/string.hh"
using namespace deweylab;
using namespace app;

class FastaReformatter : public FastaPipe {
public:
    FastaReformatter();
    bool process_record(Record& rec);
protected:
    bool strip_title;
    bool remove_gaps;
    bool remove_empty;
    bool unmask;
};

FastaReformatter::FastaReformatter() {
    // Parse command line
    parser.setDescription("Reformat fasta input");
    parser.addStoreTrueOpt(
        's', "strip-title",
        "truncate the title for each record at the first whitespace character",
        strip_title = false);
    parser.addStoreTrueOpt(
        'u', "upper",
        "uppercase sequences (unmask softmasked sequence)",
        unmask = false);
    parser.addStoreTrueOpt(
        'g', "remove-gaps",
        "remove gaps from sequences",
        remove_gaps = false);
    parser.addStoreTrueOpt(
        'e', "remove-empty",
        "remove empty records",
        remove_empty = false);
}

bool FastaReformatter::process_record(Record& rec) {
    if (strip_title) {
        rec.description = "";
    }
    if (remove_gaps) {
        rec.sequence.erase(std::remove(rec.sequence.begin(),
                                       rec.sequence.end(),
                                       '-'),
                           rec.sequence.end());
    }
    if (unmask) {
        util::string::upperString(rec.sequence);
    }

    return not (remove_empty and rec.sequence.empty());
}

int main(int argc, const char* argv[]) {
    FastaReformatter app;
    return app.main(argc, argv);
}
