/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "deweylab/app/FastaPipe.hh"
#include "deweylab/bio/translation/Translator.hh"
#include "deweylab/bio/alphabet/Nucleotide.hh"
using namespace deweylab;
using namespace app;

class FastaTranslator : public FastaPipe {
public:
    FastaTranslator() {
        parser.setDescription("Translate DNA FASTA records to protein");
        parser.addStoreOpt('f', "frame",
                           "Frame (0, 1, or 2) in which to translate",
                           frame = 0, "NUM");
        parser.addStoreOpt('t', "table",
                           "Number of the translation table to use for translation",
                           tableNum = 1, "NUM");
        parser.addStoreTrueOpt('s', "to-stop",
                               "Translate each DNA record to the first stop codon",
                               toStop = false);
    }

    void run() {
		if (frame < 0 or frame > 2) {
			throw std::runtime_error("Frame must be 0, 1, or 2");
		}
        FastaPipe::run();
    }
        
    bool process_record(Record& rec) {
		bio::translation::Translator translator(tableNum);
        rec.sequence = translator.translate(rec.sequence, frame, toStop);
        return true;
    }
    
private:
	size_t frame;
	bool toStop;
	size_t tableNum;
};

int main(int argc, const char* argv[]) {
    FastaTranslator app;
    return app.main(argc, argv);
}
